/*! \file DoubleElementResult.h 
 *  \brief: header file for double result object
 *  A double result object is just derived from a DoubleInput object, with additional time and step information.
 */


#ifndef _DOUBLEELEMENTRESULT_H_
#define _DOUBLEELEMENTRESULT_H_

/*Headers:*/
/*{{{*/
#include "../Inputs/Input.h"
#include "../../include/include.h"
class Parameters;
/*}}}*/

class DoubleElementResult: public ElementResult{

	private: 
		int    enum_type;
		double value;
		int    step;
		double time;

	public:

		/*DoubleElementResult constructors, destructors: {{{*/
		DoubleElementResult();
		DoubleElementResult(int enum_type,double value,int step,double time);
		~DoubleElementResult();
		/*}}}*/
		/*Object virtual functions definitions:{{{ */
		void  Echo();
		void  DeepEcho();
		int   Id(); 
		int   MyRank();
		int   ObjectEnum();
		Object* copy();
		/*}}}*/
		/*ElementResult virtual functions definitions: {{{*/
		ElementResult* SpawnTriaElementResult(int* indices);
		double  GetTime(void){return time;};
		int     GetStep(void){return step;};
		void    ProcessUnits(Parameters* parameters);
		int     NumberOfNodalValues(void);
		void    PatchFill(int row, Patch* patch);
		/*}}}*/
		/*DoubleElementResult management: {{{*/
		int   InstanceEnum();
		void GetVectorFromResults(Vector* vector,int* doflist,int* connectivitylist,int numdofs){_error_("not implemented");};
		void GetElementVectorFromResults(Vector* vector,int dof){_error_("not implemented");};
		/*}}}*/
};
#endif  /* _DOUBLEELEMENTRESULT_H */
