/*!\file DoubleVecParam.c
 * \brief: implementation of the DoubleVecParam object
 */

/*header files: */
/*{{{*/
#ifdef HAVE_CONFIG_H
	#include <config.h>
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include <stdio.h>
#include <string.h>
#include "../objects.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "../../Container/Container.h"
#include "../../include/include.h"
/*}}}*/

/*DoubleVecParam constructors and destructor*/
/*FUNCTION DoubleVecParam::DoubleVecParam(){{{*/
DoubleVecParam::DoubleVecParam(){
	return;
}
/*}}}*/
/*FUNCTION DoubleVecParam::DoubleVecParam(int enum_type,IssmDoubleVec values,int M){{{*/
DoubleVecParam::DoubleVecParam(int in_enum_type,double* in_values, int in_M){

	enum_type=in_enum_type;
	M=in_M;

	values=(double*)xmalloc(M*sizeof(double));
	memcpy(values,in_values,M*sizeof(double));
}
/*}}}*/
/*FUNCTION DoubleVecParam::~DoubleVecParam(){{{*/
DoubleVecParam::~DoubleVecParam(){
	xfree((void**)&values);
	return;
}
/*}}}*/

/*Object virtual functions definitions:*/
/*FUNCTION DoubleVecParam::Echo {{{*/
void DoubleVecParam::Echo(void){

	printf("DoubleVecParam:\n");
	printf("   enum: %i (%s)\n",this->enum_type,EnumToStringx(this->enum_type));
	printf("   vector size: %i\n",this->M);

}
/*}}}*/
/*FUNCTION DoubleVecParam::DeepEcho{{{*/
void DoubleVecParam::DeepEcho(void){

	int i;
	
	printf("DoubleVecParam:\n");
	printf("   enum: %i (%s)\n",this->enum_type,EnumToStringx(this->enum_type));
	printf("   vector size: %i\n",this->M);
	for(i=0;i<this->M;i++){
		printf("%i %g\n",i,this->values[i]);
	}
}
/*}}}*/
/*FUNCTION DoubleVecParam::Id{{{*/
int    DoubleVecParam::Id(void){ return -1; }
/*}}}*/
/*FUNCTION DoubleVecParam::MyRank{{{*/
int    DoubleVecParam::MyRank(void){ 
	extern int my_rank;
	return my_rank; 
}
/*}}}*/
/*FUNCTION DoubleVecParam::ObjectEnum{{{*/
int DoubleVecParam::ObjectEnum(void){

	return DoubleVecParamEnum;

}
/*}}}*/
/*FUNCTION DoubleVecParam::copy{{{*/
Object* DoubleVecParam::copy() {
	
	return new DoubleVecParam(this->enum_type,this->values,this->M);

}
/*}}}*/

/*DoubleVecParam virtual functions definitions: */
/*FUNCTION DoubleVecParam::GetParameterValue(double** pdoublearray,int* pM){{{*/
void  DoubleVecParam::GetParameterValue(double** pdoublearray,int* pM){
	double* output=NULL;
	int M;

	M=this->M;
	output=(double*)xmalloc(M*sizeof(double));
	memcpy(output,values,M*sizeof(double));

	/*Assign output pointers:*/
	if(pM) *pM=M;
	*pdoublearray=output;
}
/*}}}*/
/*FUNCTION DoubleVecParam::GetParameterValue(double** pdoublearray,int* pM){{{*/
void  DoubleVecParam::GetParameterValue(double** pdoublearray,int* pM,int* pN){
	double* output=NULL;
	int M;
	int N;

	N=1;
	M=this->M;
	output=(double*)xmalloc(M*sizeof(double));
	memcpy(output,values,M*sizeof(double));

	/*Assign output pointers:*/
	if(pM) *pM=M;
	if(pN) *pN=N;
	*pdoublearray=output;
}
/*}}}*/
/*FUNCTION DoubleVecParam::GetParameterValue(int** pintarray,int* pM){{{*/
void  DoubleVecParam::GetParameterValue(int** pintarray,int* pM){
	_error_("DoubleVec param of enum %i (%s) cannot return an array of int",enum_type,EnumToStringx(enum_type));
}
/*}}}*/
/*FUNCTION DoubleVecParam::GetParameterName{{{*/
void DoubleVecParam::GetParameterName(char**pname){
	EnumToStringx(pname,this->enum_type);
}
/*}}}*/
/*FUNCTION DoubleVecParam::SetValue{{{*/
void  DoubleVecParam::SetValue(double* doublearray,int in_M){

	/*avoid leak: */
	xfree((void**)&this->values);

	this->values=(double*)xmalloc(in_M*sizeof(double));
	memcpy(this->values,doublearray,in_M*sizeof(double));

	this->M=in_M;
}
/*}}}*/
/*FUNCTION DoubleVecParam::UnitConversion{{{*/
void  DoubleVecParam::UnitConversion(int direction_enum){
	::UnitConversion(this->values,this->M,direction_enum,this->enum_type);
}
/*}}}*/
